import { SourceEntityDescriptor, storage } from '@powersync/service-core';
import * as pgwire from '@powersync/service-jpgwire';
import { DatabaseInputRow, SqliteRow, SqlSyncRules, TablePattern } from '@powersync/service-sync-rules';
import { PgManager } from './PgManager.js';
export interface WalStreamOptions {
    connections: PgManager;
    storage: storage.SyncRulesBucketStorage;
    abort_signal: AbortSignal;
}
interface InitResult {
    /** True if initial snapshot is not yet done. */
    needsInitialSync: boolean;
    /** True if snapshot must be started from scratch with a new slot. */
    needsNewSlot: boolean;
}
export declare const ZERO_LSN = "00000000/00000000";
export declare const PUBLICATION_NAME = "powersync";
export declare const POSTGRES_DEFAULT_SCHEMA = "public";
export declare const KEEPALIVE_CONTENT = "ping";
export declare const KEEPALIVE_BUFFER: Buffer<ArrayBuffer>;
export declare const KEEPALIVE_STATEMENT: pgwire.Statement;
export declare const isKeepAliveMessage: (msg: pgwire.PgoutputMessage) => boolean;
export declare const sendKeepAlive: (db: pgwire.PgClient) => Promise<void>;
export declare class MissingReplicationSlotError extends Error {
    constructor(message: string);
}
export declare class WalStream {
    sync_rules: SqlSyncRules;
    group_id: number;
    connection_id: number;
    private readonly storage;
    private readonly slot_name;
    private connections;
    private abort_signal;
    private relation_cache;
    private startedStreaming;
    constructor(options: WalStreamOptions);
    get stopped(): boolean;
    getQualifiedTableNames(batch: storage.BucketStorageBatch, db: pgwire.PgConnection, tablePattern: TablePattern): Promise<storage.SourceTable[]>;
    initSlot(): Promise<InitResult>;
    /**
     * If a replication slot exists, check that it is healthy.
     */
    private checkReplicationSlot;
    estimatedCount(db: pgwire.PgConnection, table: storage.SourceTable): Promise<string>;
    /**
     * Start initial replication.
     *
     * If (partial) replication was done before on this slot, this clears the state
     * and starts again from scratch.
     */
    startInitialReplication(replicationConnection: pgwire.PgConnection, status: InitResult): Promise<void>;
    initialReplication(db: pgwire.PgConnection): Promise<void>;
    static getQueryData(results: Iterable<DatabaseInputRow>): Generator<SqliteRow>;
    private snapshotTable;
    handleRelation(batch: storage.BucketStorageBatch, descriptor: SourceEntityDescriptor, snapshot: boolean): Promise<storage.SourceTable>;
    private getTable;
    writeChange(batch: storage.BucketStorageBatch, msg: pgwire.PgoutputMessage): Promise<storage.FlushedResult | null>;
    replicate(): Promise<void>;
    initReplication(replicationConnection: pgwire.PgConnection): Promise<void>;
    streamChanges(replicationConnection: pgwire.PgConnection): Promise<void>;
    ack(lsn: string, replicationStream: pgwire.ReplicationStream): Promise<void>;
    /**
     * Ensures that the storage is compatible with the replication connection.
     * @throws {DatabaseConnectionError} If the storage is not compatible with the replication connection.
     */
    protected ensureStorageCompatibility(): Promise<storage.ResolvedBucketBatchCommitOptions>;
    /**
     * Check if the replication connection Postgres server supports
     * viewing the contents of logical replication messages.
     */
    protected checkLogicalMessageSupport(): Promise<boolean>;
}
export {};
